@@ -3,8 +3,10 @@
 from django.core.paginator import Paginator, InvalidPage
 from django.core.exceptions import ObjectDoesNotExist
 from django.db.models import Count, Min, Q
+from django.contrib.sessions.models import Session
 from django.utils import formats
-import os
+import os, re
+from datetime import datetime
 
 from django.contrib.auth.models import User
 from models import Song, Artist, Album, Genre, Queue, Favourite, History
@@ -37,7 +39,81 @@ class api_base:
         self.user_id = user_id
 
     def set_search_term(self, term):
-        self.search_term = term
+        options = self.parseSearchString(
+            (
+                "title",
+                "artist",
+                "album",
+                "genre",
+                "year",
+            ),
+            term
+        )
+        for key, value in options.items():
+            if key == "title":
+                self.set_search_title(value)
+            elif key == "artist":
+                self.set_search_artist_name(value)
+            elif key == "album":
+                self.set_search_album_title(value)
+            elif key == "genre":
+                try:
+                    genre = Genre.objects.all().filter(Name__iexact=value)[0:1].get()
+                    self.set_filter_genre(genre.id)
+                except ObjectDoesNotExist:
+                    pass
+            elif key == "year":
+                self.set_filter_year(value)
+
+        self.search_term = options["term"]
+
+    def parseSearchString(self, keywords, term):
+        values = {}
+        for i in range(len(keywords)):
+            do_continue = False
+            keyword = keywords[i]
+            value = None
+            pos = term.find(keyword + ":")
+            if pos != -1:
+                value_start = pos + len(keyword) + 1
+                # no brackets, search for next whitespace
+                if term[value_start:value_start + 1] != "(":
+                    value_end = term.find(" ", value_start)
+                    if value_end == -1:
+                        value_end = len(term)
+                        do_continue = True
+                    value = term[value_start:value_end]
+                # search for next closing bracket but count opened ones
+                else:
+                    i = value_start + 1
+                    bracket_count = 1
+                    while i < len(term):
+                        char = term[i:i+1]
+                        if char == "(":
+                            bracket_count+= 1
+                        elif char == ")":
+                            bracket_count-= 1
+                            if not bracket_count:
+                                value = term[value_start:i+1]
+                                continue
+                        i+= 1
+
+                    if not value:
+                        value = term[value_start:len(term)]
+                        do_continue = True
+
+            if value is not None:
+                values[keyword] = value
+            if do_continue:
+                continue
+
+        for key, value in values.items():
+            term = term.replace(key + ":" + value, "").strip()
+            if value.startswith("("):
+                values[key] = value[1:len(value)-1]
+
+        values["term"] = re.sub("\s+", " ", term)
+        return values
 
     def set_search_title(self, term):
         self.search_title = term
@@ -69,11 +145,41 @@ class api_base:
         self.order_by_direction = direction
 
     def get_default_result(self, result_type, page):
+        search = {}
+        if self.search_title is not None:
+            value = self.search_title
+            if value.find(" ") != -1:
+                value = "(" + value + ")"
+            search["title"] = value
+        if self.search_artist_name is not None:
+            value = self.search_artist_name
+            if value.find(" ") != -1:
+                value = "(" + value + ")"
+            search["artist"] = value
+        if self.search_album_title is not None:
+            value = self.search_album_title
+            if value.find(" ") != -1:
+                value = "(" + value + ")"
+            search["album"] = value
+        if self.filter_genre is not None:
+            genre = Genre.objects.all().filter(id=self.filter_genre)[0:1].get()
+            value = genre.Name
+            if value.find(" ") != -1:
+                value = "(" + value + ")"
+            search["genre"] = value
+            search["genre_id"] = genre.id
+        if self.filter_year is not None:
+            search["year"] = str(self.filter_year)
+        if self.search_term is not None:
+            search["term"] = self.search_term
+
         return {
             "type": result_type,
             "page": page,
             "hasNextPage": False,
-            "itemList": []
+            "itemList": [],
+            "order": [],
+            "search": search,
         }
 
     def result_add_queue_and_favourite(self, song, dataset):
@@ -83,6 +189,7 @@ class api_base:
                 for user in queue.User.all():
                     if user.id == self.user_id:
                         dataset["queued"] = True
+                        break
             except ObjectDoesNotExist:
                 pass
             try:
@@ -102,10 +209,30 @@ class api_base:
 
             return object_list.order_by(field_name)
         elif not self.order_by_default is None:
-            object_list = object_list.order_by(*self.order_by_default)
+            order = []
+            for key, value in self.order_by_default.items():
+                order.append(value)
+
+            object_list = object_list.order_by(*order)
 
         return object_list
 
+    def result_set_order(self, result):
+        result["order"] = []
+
+        if not self.order_by_field is None:
+            result["order"].append({
+                "field": self.order_by_field,
+                "direction": self.order_by_direction,
+            })
+        elif not self.order_by_default is None:
+            for field, order in self.order_by_default.items():
+                result["order"].append({
+                    "field": field,
+                    "direction": "desc" if order.startswith("-") else "asc",
+                })
+
+        return result
 
 class songs(api_base):
     order_by_fields = {
@@ -116,6 +243,9 @@ class songs(api_base):
         "genre": "Genre__Name",
         "length": "Length",
     }
+    order_by_default = {
+        "title": "Title",
+    }
 
     def index(self, page=1):
         object_list = Song.objects.all()
@@ -173,6 +303,7 @@ class songs(api_base):
         except InvalidPage:
             return result
 
+        result = self.result_set_order(result)
         result["hasNextPage"] = page_obj.has_next()
         for item in page_obj.object_list:
             dataset = {
@@ -226,8 +357,12 @@ class songs(api_base):
             song_instance = data.Song
             data.delete()
         except ObjectDoesNotExist:
-            song_instance = Song.objects.order_by('?')[0:1].get()
-            self.addToHistory(song_instance, None)
+            try:
+                song_instance = self.getRandomSongByPreferences()
+                self.addToHistory(song_instance, None)
+            except ObjectDoesNotExist:
+                song_instance = Song.objects.order_by('?')[0:1].get()
+                self.addToHistory(song_instance, None)
 
         # remove missing files
         if not os.path.exists(song_instance.Filename):
@@ -236,6 +371,60 @@ class songs(api_base):
 
         return song_instance
 
+    def getRandomSongByPreferences(self):
+        artists = {}
+
+        # get logged in users
+        sessions = Session.objects.exclude(
+            expire_date__gt=datetime.today()
+        )
+        for session in sessions.all():
+            data = session.get_decoded()
+            user_id = data["_auth_user_id"]
+
+            # get newest favourites
+            favourites = Favourite.objects.filter(User__id=user_id)[0:30]
+            for favourite in favourites:
+                if not artists[favourite.Song.Artist.id]:
+                    artists[favourite.Song.Artist.id] = 0
+                artists[favourite.Song.Artist.id]+= 1
+
+            # get last voted songs
+            votes = History.objects.filter(User__id=user_id)[0:30]
+            for vote in votes:
+                if not vote.Song.Artist.id in artists:
+                    artists[vote.Song.Artist.id] = 0
+                artists[vote.Song.Artist.id]+= 1
+
+        # nothing played and no favourites
+        if not len(artists):
+            raise ObjectDoesNotExist
+
+        # calculate top artists
+        from operator import itemgetter
+        sorted_artists = sorted(
+            artists.iteritems(),
+            key=itemgetter(1),
+            reverse=True
+        )[0:30]
+        artists = []
+        for key in range(len(sorted_artists)):
+            artists.append(sorted_artists[key][0])
+
+        # get the 50 last played songs
+        history = History.objects.all()[0:50]
+        last_played = []
+        for item in history:
+            last_played.append(item.Song.id)
+
+        # find a song not played recently
+        song_instance = Song.objects.exclude(
+            id__in=last_played
+        ).filter(
+            Artist__id__in=artists
+        ).order_by('?')[0:1].get()
+        return song_instance
+
     def addToHistory(self, song_instance, user_list):
         history_instance = History(
             Song=song_instance
@@ -255,11 +444,16 @@ class history(api_base):
         "genre": "Song__Genre__Name",
         "created": "Created",
     }
+    order_by_default = {
+        "created": "-Created",
+    }
 
     def index(self, page=1):
         object_list = History.objects.all()
         object_list = self.source_set_order(object_list)
-        return self.build_result(object_list, page)
+        result = self.build_result(object_list, page)
+        result = self.result_set_order(result)
+        return result
 
     def build_result(self, object_list, page):
         # prepare result
@@ -272,6 +466,7 @@ class history(api_base):
         except InvalidPage:
             return result
 
+        result = self.result_set_order(result)
         result["hasNextPage"] = page_obj.has_next()
         for item in page_obj.object_list:
             dataset = {
@@ -334,11 +529,16 @@ class history_my(history):
         "genre": "Song__Genre__Name",
         "created": "Created",
     }
+    order_by_default = {
+        "created": "-Created",
+    }
 
     def index(self, page=1):
         object_list = History.objects.all().filter(User__id=self.user_id)
         object_list = self.source_set_order(object_list)
         result = self.build_result(object_list, page)
+
+        result = self.result_set_order(result)
         result["type"] = "history/my"
         return result
 
@@ -353,10 +553,10 @@ class queue(api_base):
         "created": "Created",
         "votes": "VoteCount",
     }
-    order_by_default = [
-        "-VoteCount",
-        "MinCreated"
-    ]
+    order_by_default = {
+        "votes": "-VoteCount",
+        "created": "MinCreated",
+    }
 
     def index(self, page=1):
         object_list = Queue.objects.all()
@@ -374,6 +574,7 @@ class queue(api_base):
         except InvalidPage:
             return result
 
+        result = self.result_set_order(result)
         result["hasNextPage"] = page_obj.has_next()
         for item in page_obj.object_list:
             result["itemList"].append(self.get(item.Song.id))
@@ -469,6 +670,9 @@ class favourites(api_base):
         "genre": "Song__Genre__Name",
         "created": "Created",
     }
+    order_by_default = {
+        "created": "-Created",
+    }
 
     def index(self, page=1):
         object_list = Favourite.objects.all().filter(User__id=self.user_id)
@@ -484,6 +688,7 @@ class favourites(api_base):
         except InvalidPage:
             return result
 
+        result = self.result_set_order(result)
         result["hasNextPage"] = page_obj.has_next()
         for item in page_obj.object_list:
             result["itemList"].append(self.get(item.Song.id))
@@ -563,6 +768,9 @@ class artists(api_base):
     order_by_fields = {
         "artist": "Name",
     }
+    order_by_default = {
+        "artist": "Name",
+    }
 
     def index(self, page=1):
         # prepare result
@@ -578,6 +786,7 @@ class artists(api_base):
         except InvalidPage:
             return result
 
+        result = self.result_set_order(result)
         result["hasNextPage"] = page_obj.has_next()
         for item in page_obj.object_list:
             dataset = {
@@ -593,6 +802,10 @@ class artists(api_base):
 class albums(api_base):
     order_by_fields = {
         "album": "Title",
+        "artist": "Artist__Name",
+    }
+    order_by_default = {
+        "album": "Title",
     }
 
     def index(self, page=1):
@@ -609,6 +822,7 @@ class albums(api_base):
         except InvalidPage:
             return result
 
+        result = self.result_set_order(result)
         result["hasNextPage"] = page_obj.has_next()
         for item in page_obj.object_list:
             dataset = {
@@ -629,6 +843,9 @@ class genres(api_base):
     order_by_fields = {
         "genre": "Name",
     }
+    order_by_default = {
+        "genre": "Name",
+    }
 
     def index(self, page=1):
         # prepare result
@@ -644,6 +861,7 @@ class genres(api_base):
         except InvalidPage:
             return result
 
+        result = self.result_set_order(result)
         result["hasNextPage"] = page_obj.has_next()
         for item in page_obj.object_list:
             dataset = {
@@ -660,9 +878,9 @@ class years(api_base):
     order_by_fields = {
         "year": "Year",
     }
-    order_by_default = [
-        "Year"
-    ]
+    order_by_default = {
+        "year": "Year"
+    }
 
     def index(self, page=1):
         # prepare result
@@ -679,6 +897,7 @@ class years(api_base):
         except InvalidPage:
             return result
 
+        result = self.result_set_order(result)
         result["hasNextPage"] = page_obj.has_next()
         for item in page_obj.object_list:
             dataset = {
